<?php
/**
 *  Description of GepConvocadoreuniaoLogic
 *
 *  @package Logic.
 *
 *  @author  samuelleite
 */
class GepConvocadoreuniaoLogic{



    private $_GepConvocadoreuniaoEntity;
    private $_ConvocadoreuniaoFacade;
    private $_GerSalaEntity;


    /**
     * Metodo construtor.
     */
    public function __construct(){
        $this->_GepConvocadoreuniaoEntity = new GepConvocadoreuniaoEntity();
        $this->_GerSalaEntity             = new GerSalaEntity();
    }


    /**
     * Altera dados do convocado.
     *
     * @param integer $nratividadegerada Numero da atividade.
     * @param integer $nrorg             Numero da organizacao.
     *
     * @return boolean
     */
    public static function alteraDadosConvocado($nratividadegerada, $nrorg) {
        return GepConvocadoreuniaoEntity::alteraDadosConvocado($nratividadegerada, $nrorg);
    }


     /** ok
     * Retorna Convocados de um reuniao.
     *
     * @param integer $reuniao Numero da reuniao.
     * @param integer $nrorg   Numero da organizacao.
     *
     * @return Ambigous <<type>, multitype:, mixed>|boolean
     */
    public static function consultaConvocadosPorReuniao ($reuniao ,$nrorg) {
        try {
            $dados = GepConvocadoreuniaoEntity::consultaConvocadosPorReuniao($reuniao, $nrorg);
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()){
            case GepConvocadoreuniaoEntity::SEM_REGISTRO:
                return false;
            default:
                tk_warning('Erro desconhecido: '.$e->getMessage());
                return null;
            }
        }
    }


    /**
     * Retorna Convocado.
     *
     * @param integer $nrconvocadoreuniao Numero da reuniao.
     * @param integer $nrorg              Numero da organizacao.
     *
     * @return Ambigous <<type>, multitype:, mixed>|boolean
    */
    public static function consultaConvocadoReuniao ($nrconvocadoreuniao ,$nrorg) {
        try{
            $dados = GepConvocadoreuniaoEntity::consultaConvocadoReuniao($nrconvocadoreuniao, $nrorg);
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()){
            case GepConvocadoreuniaoEntity::SEM_REGISTRO:
                return false;
            default:
                tk_warning('erro desconhecido: '.$e->getMessage());
            }
        }
    }


    /**
     * Exclui Convocado por reuniao.
     *
     * @param integer $nrreuniao Numero da reuniao.
     * @param integer $nrorg     Numero da Organizacao.
     *
     * @return void
    */
    public static function excluiConvocadoPorReuniao($nrreuniao, $nrorg){
        GepConvocadoreuniaoEntity::excluiConvocadoPorReuniao($nrreuniao, $nrorg);
    }


    /**
     * Exclui Convocado.
     *
     * @param integer $nrconvocadoreuniao Numero da reuniao.
     * @param integer $nrorg              Numero da Organizacao.
     *
     * @return void
    */
    public static function excluiConvocado($nrconvocadoreuniao, $nrorg){
        GepConvocadoreuniaoEntity::excluiConvocado($nrconvocadoreuniao, $nrorg);
    }


    /**
     * Conta quantos parceiros foram convocados para uma reuniao.
     *
     * @param integer $nrorg     Numero da Organizacao.
     * @param integer $nrreuniao Numero da reuniao.
     *
     * @return multitype:|boolean
     */
    public static function consultaQtdParceirosConvocados($nrorg, $nrreuniao){
        $qtd = GepConvocadoreuniaoEntity::consultaQtdParceirosConvocados($nrorg, $nrreuniao);
        return $qtd;
    }


    /**
     * Metodo para gerar novo codigo de Convocado.
     *
     * @param integer $nrorg Numero da organizacao.
     *
     * @return Ambigous <$codigo, number, number, string>
     */
    public static function novoCodigoGepConvodoReuniao($nrorg){
        $codigo = GepConvocadoreuniaoEntity::retornaProxNumeroConvocadoreuniao($nrorg);
        return $codigo;
    }


    /**
     * Metodo para verificar um novo registro de convocados para reuniao.
     *
     * @param array   $v_fields          Array com os valores do campos da tab.
     * @param boolean $novoregistro      Novo registro.
     * @param integer $antigo            Valor anterior do Operador Convocado.
     * @param string  $dtinicioplanejado Data de inicio planejado.
     * @param string  $dtfimplanejado    Data de final planejado.
     * @param integer $qthorasplanejada  Quantidade de horas planejado.
     * @param string  $referencia        Referencia.
     *
     * @return number|boolean
     */
    public function verificarConvocados(array $v_fields, $novoregistro, $antigo,
        $dtinicioplanejado, $dtfimplanejado, $qthorasplanejada, $referencia){
        $operador = $v_fields['NRPARCOPERADOR'];
        $convocado = $v_fields['NMCONVOCADOREUNIAO'];
        $email = $v_fields['DSEMAILCONVOCADO'];
        $nrorg = $v_fields['NRORG'];
        if (empty($operador) === false && empty($convocado) === false) {
            if ($novoregistro === false) {
                $v_fields['NRATIVIDADEGERADA'] = null;
                $v_fields['NRPARCOPERADOR'] = null;
                $this->_GepConvocadoreuniaoEntity->insereConvocado($v_fields);
                return 1;
            }

            if (empty($antigo) === false) {
                $v_fields['NRATIVIDADEGERADA'] = null;
                $v_fields['NRPARCOPERADOR'] = null;
                $this->_GepConvocadoreuniaoEntity->insereConvocado($v_fields);
                return 1;
            } else {
                $v_fields['NMCONVOCADOREUNIAO'] = null;
                $v_fields['DSEMAILCONVOCADO'] = null;
                $v_fields['NRATIVIDADEGERADA'] = GepAtividadeLogic::retornaProxNumAtividade($nrorg);
                self::cadastraAtividadeReuniao(
                    $v_fields['NRPARCOPERADOR'], $v_fields['NRATIVIDADEGERADA'],
                    $v_fields['NRORG'], $referencia, $dtinicioplanejado,
                    $dtfimplanejado, $qthorasplanejada
                );
                $this->_GepConvocadoreuniaoEntity->insereConvocado($v_fields);
                return 1;
            }
        } else if (empty($operador) === false && empty($convocado) === true) {
            return 2;
        } else if (empty($operador) === true && empty($convocado) === false) {
            return 3;
        }

        return true;
    }


    public function incluiAtividade(
        $nratividade, $nrorg, $nratividadeproc, $nrsolicitacao, $nrparcnegosoli, $nrparcopersoli, $nrparcoperresp,
        $nrparcoperconc, $dsreferenciaativ, $idsituacaoativ, $dtinicioplanejado, $dtfimplanejado, $qthorasplanejada,
        $dtfinalizacao, $dtconclusao, $txobservatividade, $data, $cdoperador, $txatividade
    ) {
        $parametroProc = GepParamGeralGepLogic::consultaParametroPorNrorg($nrorg);
        if ($parametroProc !== false) {
            $inclui_atividade = GepAtividadeEntity::incluiAtividade(
                $nratividade, $nrorg, $parametroProc->getNrativprocreuniao(), $nrsolicitacao, $nrparcnegosoli,
                $nrparcopersoli, $nrparcoperresp, $nrparcoperconc, $dsreferenciaativ, $idsituacaoativ, $dtinicioplanejado,
                $dtfimplanejado, $qthorasplanejada, $dtfinalizacao, $dtconclusao, 'S', $txobservatividade,
                $data, $cdoperador, $txatividade
            );
        } else {
            $inclui_atividade = GepAtividadeEntity::incluiAtividade(
                $nratividade, $nrorg, $nratividadeproc, $nrsolicitacao, $nrparcnegosoli,
                $nrparcopersoli, $nrparcoperresp, $nrparcoperconc, $dsreferenciaativ, $idsituacaoativ, $dtinicioplanejado,
                $dtfimplanejado, $qthorasplanejada, $dtfinalizacao, $dtconclusao, 'S', $txobservatividade,
                $data, $cdoperador, $txatividade
            );
        }

        return true;
    }


    /**
     * Metodo para verificar a mudanca na participacao.
     *
     * @param integer $nrorg             Numero da organizacao.
     * @param integer $parceiro          Parceiro logado.
     * @param string  $situ_ant          Situacao anterior.
     * @param string  $situ_post         Situacao posterior.
     * @param integer $parcconvocado     Numero do parceiro.
     * @param string  $dtinicioplanejado Data de inicio plenejado.
     * @param string  $dtfimplanejado    Data de fim planejado.
     * @param float   $qthorasplanejada  Quantidade de horas planejadas.
     * @param string  $data              Data atual.
     * @param integer $operador          Operador logado.
     * @param string  $data_real_inicio  Data de inicio real.
     * @param string  $data_real_final   Data de fim real.
     * @param integer $atividade         Numero da atividade.
     *
     * @return string|integer
     */
    public function verificaParticipacao(
        $nrorg, $parceiro, $situ_ant, $situ_post, $parcconvocado, $dtinicioplanejado, $dtfimplanejado,
        $qthorasplanejada, $data, $operador, $data_real_inicio, $data_real_final, $atividade)
    {
        // Verifica se o parceiro convocado esta preenchido.
        if (empty($parcconvocado) === false) {
            if ($situ_post === "S") {
                // Se a participacao alterar de N - Nao - para S - Sim.
                if ($situ_ant === 'N') {
                    // Inclui atividade para o parceiro convocado.
                    $nratividade = GepAtividadeEntity::retornaProxNumAtividade($nrorg);
                    $this->incluiAtividade(
                        $nratividade, $nrorg, null, null, null, $parceiro, $parcconvocado, null, null, 'PENDENTE',
                        $dtinicioplanejado, $dtfimplanejado, $qthorasplanejada, null, null, null, $data,
                        $operador, null
                    );

                    // Inclui apropriacao para esta atividade.
                    $data_real = new TkDateTime($data_real_inicio);
                    $dtfimreal = $data_real->addSecond(1);

                    $insere_apropriacao = GepApropriacaoEntity::insereApropriacao(
                        $nratividade,
                        $dtinicioplanejado,
                        $dtfimplanejado,
                        $data_real_inicio,
                        $dtfimreal->toString()
                    );

                    return $nratividade;

                    // Se vier de P - Pendente - para S - Sim.
                } else {
                    // Consulto se houve apropriacao para a atividade da reuniao.
                    $verif_apropriacao = GepApropriacaoEntity::consultaDadosApropriacaoByAtividade(
                        $nrorg, $atividade
                    );

                    // Se nao tiver...
                    if ($verif_apropriacao === false) {
                        // Verifica se houve alguma apropriacao no periodo em que ocorreu a reuniao.
                        $apr_periodo = GepApropriacaoEntity::consultaDadosApropriacaoByPeriodoDeDatas(
                            $parcconvocado, $data_real_inicio, $data_real_final, $nrorg
                        );

                        // Se sim...
                        if ($apr_periodo !== false) {
                            // Atualizacao da apropriacao existente no mesmo periodo.
                            // Adiciona 1 hora na data inicial real da apropriacao e atualizo.
                            $dtinicioaprreal = $apr_periodo->getDtinicioaproprireal();
                            $data_aprreal = new TkDateTime($dtinicioaprreal);
                            $data_aprfimreal = $data_aprreal->addHour(1);

                            $atualiza_apr_periodo = GepApropriacaoEntity::atualizaDtRealApropriacao(
                                $parcconvocado, $apr_periodo->getNratividade(), $dtinicioaprreal,
                                $data_aprfimreal->toString(), $data, $nrorg, $operador, $nrorg
                            );

                            // Adiciono um segundo na hora final real da apropriacao existente para gerar a inicial
                            // da atividade da reuniao.
                            $data_real = new TkDateTime($data_aprfimreal->toString());
                            $dtinicioreal = $data_real->addSecond(1);

                            // E mais um segundo no total para gerar a hora final da atividade da reuniao.
                            $data_real = new TkDateTime($dtinicioreal->toString());
                            $dtfimreal = $data_real->addSecond(1);

                            // Insiro uma apropriacao para a atividade da reuniao com as datas geradas.
                            $insere_apropriacao = GepApropriacaoEntity::insereApropriacao(
                                $atividade,
                                $dtinicioplanejado,
                                $dtfimplanejado,
                                $dtinicioreal,
                                $dtfimreal->toString()
                            );

                            // Insiro a outra parte da apropriacao que existia.
                            // Com hora inicial real de um segundo depois da apr da reuniao e data final real
                            // original.
                            $data_inicioaprreal = new TkDateTime($dtfimreal->toString());
                            $data_real_inicio = $data_inicioaprreal->addSecond(1);

                            $insere_apr_periodo = GepApropriacaoEntity::insereApropriacao(
                                $apr_periodo->getNratividade(),
                                $apr_periodo->getDtinicioapropriacao(),
                                $apr_periodo->getDtfimapropriacao(),
                                $data_real_inicio->toString(),
                                $apr_periodo->getDtfimaproprireal()
                            );
                            // Se nao tiver apropriacao para atividade ele insere com intervalo de 1s.
                        } else {
                            $data_inicio = new TkDateTime($data_real_inicio);
                            $data_real_final = $data_inicio->addSecond(1);
                            $insere_apr_periodo = GepApropriacaoEntity::insereApropriacao(
                                $atividade,
                                $dtinicioplanejado,
                                $dtfimplanejado,
                                $data_real_inicio,
                                $data_real_final->toString()
                            );
                        }

                        // Se tiver prossegue a funcao.
                    }

                    return $atividade;
                }
            } elseif ($situ_post === "N") {
                // Verifica se ha apropriacao para a atividade.
                $verif_apropriacao = GepApropriacaoEntity::consultaDadosApropriacaoByAtividade(
                    $nrorg, $atividade
                );

                // Se tiver, exclui.
                if ($verif_apropriacao !== false) {
                    $exclui_apr = GepApropriacaoEntity::excluiApropriacaoByAtividade($nrorg, $atividade);
                }

                // Atualizacao do campo de atividade gerada do convocado para e exclusao da atividade.
                $atual_convocado = GepConvocadoreuniaoEntity::alteraDadosConvocado($atividade, $nrorg);
                $exclui_ativ = GepAtividadeEntity::excluiAtividade($nrorg, $atividade);

                return $situ_post;
                // Se situacao posterior for igual a P.
            } else {
                return $situ_post;
            }
        }

        return null;
    }


    /**
     * Metodo para remover o NROPERADOR.
     *
     * @param integer $nrorg             Numero da organizacao.
     * @param integer $nratividadegerada Numero da atividade.
     *
     * @return boolean
     */
    public function atualizaOperadorByAtividade($nrorg, $nratividadegerada){
        return $this->_GepConvocadoreuniaoEntity->atualizaOperadorByAtividade($nrorg, $nratividadegerada);
    }


    /**
     * Atualiza/Insere Atividade.
     *
     * @param integer $operador          Numero do operador responsavel.
     * @param integer $atividade         Numero da atividade.
     * @param integer $nrorg             Numero da organizacao.
     * @param integer $referencia        Referencia da atividade.
     * @param string  $dtinicioplanejado Data de inicio planejado.
     * @param string  $dtfimplanejado    Data de fim planejado.
     * @param float   $qthorasplanejada  Quantidade de horas planejado.
     *
     * @return void
     */
    public function cadastraAtividadeReuniao($operador, $atividade, $nrorg, $referencia,
                                             $dtinicioplanejado, $dtfimplanejado,
                                              $qthorasplanejada){
        $atividadeProc = GepParamGeralGepLogic::consultaParametroPorNrorg($nrorg);
        $logado = Ambiente::getCdOperador();
        if (is_object($atividadeProc) === false) {
                 GepAtividadeLogic::salvaAtividade(
                     $atividade, $nrorg, null,
                     null, null, $logado,
                     $operador, null, $referencia,
                     'PENDENTE', $dtinicioplanejado, $dtfimplanejado,
                     $qthorasplanejada, null, null,
                     'S', null, now(), Ambiente::getCdoperador(), null, now(),
                     Ambiente::getParceiroNegocio()
                 );
        } else {
                 GepAtividadeLogic::salvaAtividade(
                     $atividade, $nrorg, $atividadeProc->getNrativprocreuniao(),
                     null, null, $logado,
                     $operador, null, $referencia,
                     'PENDENTE', $dtinicioplanejado, $dtfimplanejado,
                     $qthorasplanejada, null, null,
                     'S', null, now(), Ambiente::getCdoperador(), null, null,
                     Ambiente::getParceiroNegocio()
                 );
        }
    }


    /**
     * Verifica capacidade restante na sala.
     *
     * @param integer $nrorg    Numero da organizacao.
     * @param integer $sala     Numero da sala.
     * @param string  $dtinicio Data de inicio.
     * @param string  $dtfim    Data de fim.
     *
     * @return integer
     */
    public function calculaQtdRestanteConvocado($nrorg, $sala, $dtinicio, $dtfim) {
        if (empty($sala) === false) {
            $qt_convocados = $this->_GepConvocadoreuniaoEntity->consultaTotalConvocadosSalaPorPeriodo(
                $nrorg, $sala, $dtinicio, $dtfim
            );
            $qt_capacidade = $this->_GerSalaEntity->consultaDadosSala($nrorg, $sala);
            $total_restante = $qt_capacidade->getQtmaximapessoas() - $qt_convocados;
            return $total_restante;
        }
    }


}